/* Sample Verilog HDL program */
`timescale 1ns / 1ps   // use for ISM or iverilog
// For ISM make test_mynand the top level entiriy
// For Quartus waveform simulator use mynand as top level
// For iverilog test_mynand will run and envoke the display calls.

//----------- positive edged 8-bit counter  ------------//

module mycounter (
       count_out, // 8 bit vector output of the counter (output)
       clk ,      // Clock input of the design
       reset,     // active high, synchronous Reset input
       enable     // Active high enable signal for counter
       ); // End of port list

       // Outputs:
       output [7:0] count_out; // 8-bit counter output
       
       // Inputs:
       input clk;
       input reset;
       input enable;
       
       // Data types
       // All the input ports are wires   
       wire clk;
       wire reset;
       wire enable;
       // All the outputs are registers
       reg [7:0] count_out;
       
       // trigger on positive edge of the clock
       always @ (posedge clk)
        begin : MYCOUNTER // name of this block
         // check for a reset
         if (reset == 1'b1)
           begin
            count_out <= #1 8'b00000000;   // #1  == 1 unit delay
           end
         // Increment counter when enable is high
         else if (enable == 1'b1)
           begin
            count_out <= #1 count_out + 1;
           end
        end // End of MYCOUTNER Block

endmodule // end module mycounter


/* Create a test bench for mycounter -- use for ISIM or iverilog */
module test_mycounter (test_out); // no ports defined
  // defined registers and wire
  output [7:0] test_out;
  reg clk;
  reg test_reset;
  //input clk1;
  
  // instantiate a counter
  mycounter mycnt(test_out,clk,test_reset,1);

  //Behavioral code block generates stimulus to test circuit
  initial 
    begin
      // Do a reset
      clk = 1'b0; test_reset = 1'b1;
      #50 $display("out=%d, clk=%b, reset=%b", test_out, clk, test_reset);
      clk = 1'b1; test_reset = 1'b1;
      #50 $display("out=%d, clk=%b, reset=%b", test_out, clk, test_reset);
      clk = 1'b0; test_reset = 1'b1;
      #50 $display("out=%d, clk=%b, reset=%b", test_out, clk, test_reset);
      clk = 1'b0; test_reset = 1'b0;
      #50 $display("out=%d, clk=%b, reset=%b", test_out, clk, test_reset);
      
      // Give a few clocks with reset low
      clk = 1'b1; 
      #50 $display("out=%d, clk=%b, reset=%b", test_out, clk, test_reset);
      clk = 1'b0; 
      #50 $display("out=%d, clk=%b, reset=%b", test_out, clk, test_reset);
      clk = 1'b1; 
      #50 $display("out=%d, clk=%b, reset=%b", test_out, clk, test_reset);
      clk = 1'b0; 
      #50 $display("out=%d, clk=%b, reset=%b", test_out, clk, test_reset);
      
    end
    
endmodule

